%
% File: invpend.m
% Inverted pendulum simulation with visualization
% and mouse input.
% Created by Marc Bodson. Last revised: 01/18/03.
% Revised by Taylor Meacham and Ronnie Boutte Last revised: 02/09/08
% Revised by Arn Stolp Last revised: 4/2/13
% Revised again by Arn Stolp : 4/10/19

% pendulum total length (m) ADJUST LENGTH HERE
Lpend = input('Enter the pendulum length in meters (suggest between 2 and 6)');
cheat=1;    %    <= ADJUST Gravity CHEAT HERE (should stay at cheat=1)

g=9.81/cheat;          % acceleration of gravity (m/s^2)


% Set limits for simulation
thmax=pi/6;xmax=Lpend; % max. pendulum angle (rad) and cart position (m)
theta=0;xcart=0;       % initial pendulum angle (rad) and cart position (m)
%theta=5*pi/180;        % use to set an initial pendulum angle of 5 deg

% define model
dt=0.05;               % sampling period (s) -> 20Hz
a2=3*g/(2*Lpend);nump=[-a2/g 0 0];denp=[1 0 -a2];sysp=tf(nump,denp);
[ap,bp,cp,dp]=ssdata(c2d(sysp,dt,'zoh'));f=5;
icca=inv([cp;cp*ap]);iccb=icca*([dp;cp*bp+dp]); % zero velocity initialization
xp=icca*[theta;theta]-iccb*xcart;xss=icca*[thmax;thmax];
% 
% Initialization - Controller
%
mode = input('Type 1 for manual control, 2 for automatic control:\n');
switch mode;
   case 1;gmanual=xmax;    % gain of manual controller
   case 2;invpendcinit;    % initialization of automatic controller
end;
%
% Initialization - Visualization
% Adjust the position & size of the simulation window below 
% to fit the screen and to square the axes (if needed)
% Format: [position from left, position from bottom, width, height]
%
figure(1);
wpend=0.04;vgap=0.005*Lpend;wtrack=2.12*Lpend;
htrack=0.03*Lpend; %wcart=0.12*Lpend;hcart=0.05*Lpend;
wcart=0.24;hcart=0.1;
axis([-0.75*wtrack 0.75*wtrack 0 htrack+hcart+1.2*Lpend]);clf
axis([-0.75*wtrack 0.75*wtrack 0 htrack+hcart+1.2*Lpend]);
set(1,'pos',[20 300 970 393]);hold on % position/size of sim. window
xdtrack=[wtrack/2;wtrack/2;-wtrack/2;-wtrack/2];
ydtrack=[0;htrack;htrack;0];
ptrack=fill(xdtrack,ydtrack,'green'); 
xline1=[-(Lpend/2),-(Lpend/2)];yline1=[0,htrack];
pline1=plot(xline1,yline1,'black','EraseMode','background');
xline2=[(Lpend/2),(Lpend/2)];yline2=[0,htrack];
pline2=plot(xline2,yline2,'black','EraseMode','background');
xdcart=[wcart/2;wcart/2;-wcart/2;-wcart/2];
ydcart=[0;hcart;hcart;0];
pcart=fill(xdcart,ydcart+htrack+vgap,'blue','EraseMode','background'); 
xdpend=[wpend/2;wpend/2;-wpend/2;-wpend/2];
ydpend=[0;Lpend;Lpend;0];
ppend=fill(xdpend,ydpend+htrack+vgap+vgap+hcart,'red','EraseMode','background'); 
%
% Real-time simulation
%
t=0;tm=0;nt=0;done=0;tic;
while (done==0);
   while tm<nt;tm=toc;end;t=nt;nt=t+dt;
%
% Controller
% Control signal:   xref, reference cart position (m)
% Useful variables: xcart, cart position (m)
%                   theta, pendulum angle (rad)
%                   t, time (s)
%
switch mode;
   case 1; %manual control
       monitor = get(0,'MonitorPosition');  % get monitor size done above
       cursor = get(0,'PointerLocation');   % get mouse pointer position on monitor
       sensitivity=15/Lpend;                % set sensitivity to mouse movement
       position=sensitivity*((cursor(1)-1)/(monitor(1,3)-1)-0.5);  % scale
       xcom=gmanual*position;

   case 2;% automatic control
       invpendc; 
end;
%
% Inverted pendulum model
%
xcart=xcart+dt*f*(xcom-xcart);
%xcart=xcom;
if (xcart>xmax);xcart=xmax;end;
if (xcart<-xmax);xcart=-xmax;end;
theta=cp*xp+dp*xcart;xp=ap*xp+bp*xcart;
if (theta>thmax);theta=thmax;xp=xss-iccb*xcart;end;
if (theta<-thmax);theta=-thmax;xp=-xss-iccb*xcart;end;
%
% Visualization
%
u=[cos(theta) sin(theta);-sin(theta) cos(theta)];
xpcart=xdcart+xcart;ypcart=htrack+vgap+ydcart;
dpend=[xdpend ydpend]*u';
xppend=xcart+dpend(:,1);yppend=htrack+hcart+vgap+vgap+dpend(:,2);
set(pcart,'Xdata',xpcart,'Ydata',ypcart);
set(ppend,'Xdata',xppend,'Ydata',yppend);
drawnow;
%
end
hold off